﻿package de.davidochmann.utilities.doconvert
{
	/**
	 * DoConvert
	 * @version v0.70
	 * @author 	David Ochmann
	 * 
	 * @mail	mail@davidochmann.de 	
	 * @website www.davidochmann.de
	 * 
	 * 
	 * v0.60
	 * - Removed a bug in the StringToObject.as and DoGesture.as that prevented DoConvert to parse 
	 *   Strings including breaks and/or tabs.
	 * - Various code optimizations.
	 * 
	 * v0.62
	 * - Removed the NaN compiler error.
	 * - Various code optimizations.
	 * 
	 * v0.70
	 * - Added Actionscript 2 version
	 * - Reduced Filesize by removing the classes ToArray.as, ToObject.as and Type.as.
	 * - Various code optimizations.
	 * 
	 */
	 
	public class DoConvert 
	{
		private static var BOOLEAN:String = "Boolean";
		private static var OBJECT:String  = "Object";
		private static var ARRAY:String	  = "Array";
		private static var STRING:String  = "String";
		private static var INT:String	  = "int"; 
		private static var UINT:String 	  = "uint";
		private static var NUMBER:String  = "Number";
		
		
		
		public static function toArray(pString:String):Array
		{
			var str:String = pString;
			var stl:uint   = str.length;
			var rep:uint   = 0;
			var chr:String = "";
			var out:String = "";
			
			str = str.slice(1, -1);
		
			for(var l:uint = 0; l < stl; ++l)
			{
				chr = str.charAt(l);
				
				if(chr == "\'") rep++;
				if(rep % 2 == 1) {out += chr; continue;}
				if(chr != " " && chr != "\n" && chr != "\r" && chr !="\nr" && chr.charCodeAt(0) != 9) out += chr;
			}
			

			var all:uint   = out.length;
			var nes:String = "";
			
			rep = 0;
			chr = "";
			
			for(var j:uint = 0; j < all; ++j)
			{
				chr = out.charAt(j);
				
				if(chr == "[" || chr == "{") rep++;
				if(chr == "]" || chr == "}") rep--;
				
				if(chr == "," && rep != 0) nes += "ϖ±";
				else nes += chr; 
			}
			

			var lis:Array = nes.split(",");
			var lil:uint  = lis.length;
			
			for(var k:uint = 0; k < lil; ++k) lis[k] = String(lis[k]).split("ϖ±").join(",");
			
			
			var arr:Array = new Array();
			var val:String;
			
			for(var i:uint = 0; i < lil; ++i)
			{
				val = lis[i];
				
				switch(type(val))
				{
					case BOOLEAN:  arr.push((val == "true") ? true : false); break;
					case OBJECT:   arr.push(toObject(val));					 break;
					case ARRAY:    arr.push(toArray(val));	 				 break;
					case STRING:   arr.push(String(val.slice(1, -1))); 	  	 break;
					case INT:	   arr.push(int(val));						 break;
					case UINT:	   arr.push(uint(val));						 break;
					case NUMBER:   arr.push(Number(val)); 				   	 break;
				}
			}
			
			return arr;
		}
		
		
		
		public static function toObject(pString:String):Object
		{
			var str:String = pString;
			var stl:uint   = str.length;
			var rep:uint   = 0;
			var chr:String = "";
			var out:String = "";
			
			str = str.slice(1, -1);
		
			for(var l:uint = 0; l < stl; ++l)
			{
				chr = str.charAt(l);
				
				if(chr == "\'") rep++;
				if(rep % 2 == 1) {out += chr; continue;}
				if(chr != " " && chr != "\n" && chr != "\r" && chr !="\nr" && chr.charCodeAt(0) != 9) out += chr;
			}
			

			var all:uint   = out.length;
			var nes:String = "";
			
			rep = 0;
			chr = "";
			
			for(var j:uint = 0; j < all; ++j)
			{
				chr = out.charAt(j);
				
				if(chr == "[" || chr == "{") rep++;
				if(chr == "]" || chr == "}") rep--;
				
				if(chr == "," && rep != 0) nes += "ϖ±";
				else nes += chr; 
			}
			

			var lis:Array = nes.split(",");
			var lil:uint  = lis.length;
			
			for(var k:uint = 0; k < lil; ++k) lis[k] = String(lis[k]).split("ϖ±").join(",");
			
			
			var obj:Object = new Object();
			var itm:String;
			var idf:String;
			var val:String;
						
			for(var i:uint = 0; i < lil; ++i)
			{
				itm = String(lis[i]);
				idf = itm.split(":")[0];
				val = itm.split(":").slice(1).join(":");

				switch(type(val))
				{
					case BOOLEAN: obj[idf] = (val == "true") ? true : false; break;
					case OBJECT:  obj[idf] = toObject(val);					 break;
					case ARRAY:   obj[idf] = toArray(val);					 break;
					case STRING:  obj[idf] = String(val.slice(1, -1));	 	 break;
					case INT:     obj[idf] = int(val);						 break;
					case UINT:	  obj[idf] = uint(val);					  	 break;
					case NUMBER:  obj[idf] = Number(val); 				   	 break;
				}
			}
			
			return obj;
		}
		
		
		
		public static function type(pString:String):String
		{
			var typ:String;

			if(pString == "true" || pString == "false")								    {typ =  BOOLEAN; return typ;}
			if(pString.charAt(0) ==  "{" && pString.charAt(pString.length - 1) ==  "}") {typ =   OBJECT; return typ;}
			if(pString.charAt(0) ==  "[" && pString.charAt(pString.length - 1) ==  "]") {typ =    ARRAY; return typ;}
			if(pString.charAt(0) == "\"" && pString.charAt(pString.length - 1) == "\"") {typ =   STRING; return typ;}
			if(pString.charAt(0) ==  "'" && pString.charAt(pString.length - 1) ==  "'") {typ =   STRING; return typ;}
			if(int(pString)  == Number(pString) && Number(pString) < 0)					{typ =      INT; return typ;}
			if(uint(pString) == Number(pString) && Number(pString) > 0)					{typ =     UINT; return typ;}			
			if(!isNaN(Number(pString))) 												{typ =   NUMBER; return typ;}
			
			return typ;	
		}
	}
}
